/*****************************************************************************************
* File name: Rc_Sel.c
* Purpose:  Acknowledges the Reception of the Raw Channel Selection Command.			 
*
* Description:	This Program Acknowledges the Reception of the Raw Channel
*				Selection Command and Updates the Nb of Selected Channel by
*               the Attached Parameters.
*****************************************************************************************/

#ifdef SIMULATOR
#include "virtuosoSim.h"
#else // SIMULATOR
#include "node1.h"
#include "k_struct.h"
#endif

#include "spulib.h"

/*=================================================================================================*/
/*Global variables declared in this file
===================================================================================================*/

RAW_CHANNEL_SELECTION RawChannelSelectionSpectro;
RAW_CHANNEL_SELECTION RawChannelSelectionPhoto;

/*================================================================================================*/

/*
This function handles the select raw channels command.
*/
void PerformRawChannelSelection(u4 buf[])
{
	struct FIFO_TRANSMIT_MSG fifoTransmitMsg;	//msg to communicate with the data tx task
	u4 sid=buf[1]&0xFFFF;
	
	/*=============================================================================================*/
	
	if (sid!=5) /* Negative Acknowledgment:  Invalid Structure ID*/
	{
		fifoTransmitMsg.Header = K_NACK_PERFORM | K_INVALID_SID;
	  	fifoTransmitMsg.Block1.NackParameter = sid;

		KS_FIFOPut(FIFO_TRANSMIT, &fifoTransmitMsg); /* Send the Response Packet to the DPU */
	}
	else if ((buf[3]!=1)&&(buf[3]!=2)) /* Invalid SID Parameter: Observing Mode*/
	{
		fifoTransmitMsg.Header = K_NACK_PERFORM | K_INVALID_SID_PARAMETER;
	  	fifoTransmitMsg.Block1.NackParameter = buf[3];

		KS_FIFOPut(FIFO_TRANSMIT, &fifoTransmitMsg); /* Send the Response Packet to the DPU */
	}
	else if(buf[4]>31) /* Invalid SID Parameter: RCNB */
	{
		fifoTransmitMsg.Header = K_NACK_PERFORM | K_INVALID_SID_PARAMETER;
	  	fifoTransmitMsg.Block1.NackParameter = buf[4];

		KS_FIFOPut(FIFO_TRANSMIT, &fifoTransmitMsg); /* Send the Response Packet to the DPU */
	}
	else /*Positive Acknowledgment */		
	{
		switch (buf[3])
		/* Switch Observing Mode */					
		{
		case K_SPECTRO :   /* Spectroscopy */ 
			{
				RawChannelSelectionSpectro.NbSelectedRawChannels = buf[4];
				RawChannelSelectionSpectro.IndexOfFirstChannel = buf[5]-1;
				if ((buf[5]>449)||(buf[5]<1))
					RawChannelSelectionSpectro.IndexOfFirstChannel = 0;
			}break;
		case K_PHOTO :   /* Photometry */ 
			{
				RawChannelSelectionPhoto.NbSelectedRawChannels = buf[4];
				RawChannelSelectionPhoto.IndexOfFirstChannel = buf[5]-1;
				if ((buf[5]>511)||(buf[5]<1))
					RawChannelSelectionPhoto.IndexOfFirstChannel = 0;
			}break;		
		}
		fifoTransmitMsg.Header = K_PACK_PERFORM;

		KS_FIFOPut(FIFO_TRANSMIT, &fifoTransmitMsg); /* Send the Response Packet to the DPU */
	}
}
