/****************************************************************************************
* File name: Spu_tst.c
* Purpose:  Acknowledges the Command for the SPU Test			 
*
* Description:	This Program Acknowledges the Reception of the SPU Test for
*				Photometry or Spectroscopy Commands and starts the Appropriate Program 
*				in Case of Command Consistency.
*****************************************************************************************/

#ifdef SIMULATOR
#include "virtuosoSim.h"
#else // SIMULATOR
#include "node1.h"
#include "k_struct.h"
#endif
#include "spu_io.h"

/*
This function handles the SPU test command
*/
void PerformSpuTestMode(u4 buf[])
{
	struct FIFO_TRANSMIT_MSG fifoTransmitMsg;	//msg to communicate with the data tx task
	u4 sid = buf[1]&0xFFFF;

	/*=============================================================================================*/
	
	if (sid!=0) /* Negative Acknowledgment:  Invalid Structure ID*/
	{
		fifoTransmitMsg.Header = K_NACK_PERFORM | K_INVALID_SID ;
	  	fifoTransmitMsg.Block1.NackParameter = sid;
		
		KS_FIFOPut(FIFO_TRANSMIT, &fifoTransmitMsg); /* Send the Response Packet to the DPU */
	}
	else /* Positive Acknowledgment */		
	{
		fifoTransmitMsg.Header = K_PACK_PERFORM;

		/* Send the Response Packet to the DPU */
		KS_FIFOPut(FIFO_TRANSMIT, &fifoTransmitMsg); 

		/* To Signal the Start of the SPU HLSW Test */
		DataGeneratorRunning=K_YES;	
		CompressionSoftwareRunning=K_YES;
		
		if(((buf[1]>>16)&0xF)==0xA)
		{
			/* Call Data Generator of Photometry' Frames */
			KS_SemaSignal(SEMA_SPUTESTP); 
		}
		else
		{
			/* Call Data Generator of Spectroscopy' Frames */
			KS_SemaSignal(SEMA_SPUTESTS); 
		}
	}
}
