/****************************************************************************************
* File name: Spvs_Tst.c
* Version.Revision: HPSPU-HLSW-OB-V11R7
*
* Purpose: Gives the Data and the Compression Mode.
*
*					
* Private Functions: None
*
* Description:	The Supervisor For SPU Test is the Controller of the Application
*				Software while DEC/MEC ia Simulated by Data Generated On-Board SPU.
*				It Waits the SPU Test Command to Start the Compression of Data 
*				Simulated By Data Generators. It is Identical to Supervisor Task
*
* Creation date & author: 28.03.2001 by  Ahmed Nabil Belbachir
* Version, Update date & author: HPSPU-HLSW-OB-V11R7 28.07.2005 by Ahmed Nabil Belbachir
****************************************************************************************/

/* The Header Files 
===================================================================================================*/

#ifdef SIMULATOR
#include "virtuosoSim.h"
#else // SIMULATOR
#include "node1.h"
#include "k_struct.h"
#endif
#include "spuAsw.h"
#include "spulib.h"

/*=================================================================================================*/
/*Declaration of the Used Global Variables
===================================================================================================*/

extern u4 crdc_4seconds;

struct SCIENCE_PACKET TestSciencePacket;

void SupervisorTest()
{
#if 0
	/* Declaration of Local Variables
	===============================================================================================*/
	struct SCIENCE_PACKET sciencePacket;    
	struct PACKET_READY supervisorTestFifoMsg;
	u4 currentCompressionMode = K_CM_UNDEFINED;


	/*=============================================================================================*/
	while(1)
	{
		KS_SemaReset(SPU_SPVS_TST);
		KS_SemaTestW(SPU_SPVS_TST);

		while (DataGeneratorRunning==K_YES)
		{		
			/* Listen to the Data Generator
 			==========================================================================================*/
	
			KS_FIFOGetW(FIFO_SPU_TEST, &supervisorTestFifoMsg); /* Send the Response Packet to the DPU */
			MemCopyDmToDm((u4*)(&TestSciencePacket),(u4*)(&sciencePacket), 272);	

			switch(sciencePacket.Type)
			{
				case K_SPECTRO : /* Spectroscopy Mode */
				{  				
					if((FrameCounter==0) && ((sciencePacket.CurrentReadoutCounter&0x1FF)==((sciencePacket.Block2.ReadoutsPerRamps-1)&0x1FF)))  /* Synchronize the Ramp Begin with the Fill Buffer (Used at Begin of each Observing Mode) */
					{	
						/* Initialize those Parameters At Begin of Each Buffer*/
						StoreNow = K_YES;

						/* Store the Relevant Information of DEC/MEC Header from 1st Frame */
						currentCompressionMode = sciencePacket.CompressionMode;
						CurrentObsId = sciencePacket.ObsId; /* Get Initial Value of OBS ID */
					}
					/* Start Storing Data on Memory if Synchronized */
					if(StoreNow==K_YES)
					{
						/* Call the Circular Buffer for the DMC Header */
						WriteInCircularBuffer(K_CBUF_HEADER, (u4 *)(&sciencePacket)+2, 13);
						
						/* Call the Circular Buffer to Fill the Science Data */
						WriteInCircularBuffer(K_CBUF_SCIENCE_SPECTRO, sciencePacket.Data, K_SPECTRO_FRAME_SIZE_IN_WORDS);

						/* Increment the Frame Counter*/
						FrameCounter++;

						/* Start Compression after Receiving 512 Frames or While Label is Signaled*/
						if(FrameCounter == K_NB_SPECTRO_FRAMES_IN_2SEC_BUF)
						{
							/* Initialize the Buffer to Start Storage with 1st Sample in Ramp */
							StoreNow=K_NO;
							
							/* Get Initial Value of OBS ID */
							Mode.ObsId = CurrentObsId;						
							
							/*Store the Number of Readouts per Ramp*/
							if(sciencePacket.Block2.ReadoutsPerRamps==0) 
							{
								Mode.ReadoutsPerRamps =32;
							}
							else 
							{
								Mode.ReadoutsPerRamps = sciencePacket.Block2.ReadoutsPerRamps;
							}

							Mode.DetectorType = sciencePacket.Type; /* Observing Mode */
							
							/*Store the Compression Mode and Reset it*/
							if(FrameCounter <= K_SPECTRO_MINIMUM_NB_FRAMES_TO_START_PROCESSING) /* Check Frames with Buffer Granularity */
							{
								Mode.CompressionMode = K_CM_BUFFER_UNDER_GRANULARITY;
							}
							else
							{
								Mode.CompressionMode = currentCompressionMode;
							}

							/* Reset the Frames Counter */
							FrameCounter=0;
							
							if (currentCompressionMode == K_CM_SPECTRO_4_SEC_RESET)	//record if we are in the first/second half of the 4sec ramp
							{
								crdc_4seconds = sciencePacket.CurrentReadoutCounter;
							}

							if ((BufferTransmissionModeEnabled == K_NO)&&(DataGeneratorRunning==K_YES))
							{
								/* Increments the PIX Counter */
								Mode.PacketIndex++;
								KS_SemaSignal(SEMA_ASW); /* Start Compression of Buffer*/
						
							}
						}
					}
				};break;
				case K_PHOTO :  /* Photometry Mode */
				{
					if((FrameCounter==0) && ((sciencePacket.Block3.DataBlockId==1)||(sciencePacket.Block3.DataBlockId==5))) /* Synchronize the Chopper Plateau 
																									 with the First Sample (Used at Begin
																								 of each Observing Mode) */
					{	
						/* Initialize those Parameters At Begin of Each Buffer*/
						StoreNow = K_YES;
						
						/* Store the Relevant Information of DEC/MEC Header from 1st Frame */
						currentCompressionMode = sciencePacket.CompressionMode;
						CurrentObsId = sciencePacket.ObsId; /* Get Initial Value of OBS ID */
						Mode.DataBlockId=sciencePacket.Block3.DataBlockId;
					}

					if (StoreNow==K_YES)
					{

						/* Check and Store DMCH Only One Time as it is Identical for all Sub-Arrays*/
						if((sciencePacket.Block3.DataBlockId==1)||(sciencePacket.Block3.DataBlockId==5)) /* Check and Store DMCH Only One Time as it is Identic to all Sub-Arrays*/
						{
							/* Call the Circular Buffer for the DMC Header */
							WriteInCircularBuffer(K_CBUF_HEADER, (u4 *)(&sciencePacket)+2, 13);
						}
						/* Call the Circular Buffer to Fill the Science Data */
						WriteInCircularBuffer(K_CBUF_SCIENCE_BLUE_PHOT1 + sciencePacket.Block3.DataBlockId - 1, sciencePacket.Data, K_NB_PIXELS_IN_PHOTO_ARRAY);
								
						/* Get the New Frame */
						FrameCounter++;   
							
						/* Start Compression after Receiving 480 Frames */	
						if(FrameCounter == K_NB_PHOTO_FRAMES_IN_3SEC_BUF)
						{
							/* Initialize the Buffer to Start Storage with 1st Sample in Ramp */
							StoreNow=K_NO;
							
							/* Get Initial Value of OBS ID */
							Mode.ObsId = CurrentObsId;						
							
							/*Store the Observing Mode */
							Mode.DetectorType = sciencePacket.Type; 

							
							/*Store the Compression Mode and Reset it*/
							if(FrameCounter <= K_PHOTO_MINIMUM_NB_FRAMES_TO_START_PROCESSING) /* Check Frames with Buffer Granularity */
								Mode.CompressionMode = K_CM_BUFFER_UNDER_GRANULARITY;
							else
								Mode.CompressionMode = currentCompressionMode;

							if(sciencePacket.Block3.DataBlockId!=5)
								FrameCounter = FrameCounter >> 2;

							/* Store the Size of the Buffered Data for Science and DMCH */
/*							DmcHeaderBufferSizeInWords = FrameCounter *13;
							ScienceBufferSizeInWords = FrameCounter * K_NB_PIXELS_IN_PHOTO_ARRAY;
							NbFramesInBuffer = FrameCounter;*/

							FrameCounter=0;		/* Reset the Frames Counter */

							if ((BufferTransmissionModeEnabled == K_NO)&&(DataGeneratorRunning==K_YES))
							{
							/* Increments the PIX Counter */
								Mode.PacketIndex++;
								KS_SemaSignal(SEMA_ASW); /* Start Compression of Buffer*/
							}
						}
					}
				};break;
				default: /*Invalid Data Type*/
				{
					Mode.CompressionMode=currentCompressionMode;				/* Get the Compression Mode*/
					Mode.ObsId=sciencePacket.ObsId;	/* Get Initial Value of OBS ID */
					Mode.DetectorType=sciencePacket.Type;	/* Observing Mode */
					FrameCounter=0;						/* Reset the Frames Counter */
					KS_SemaSignal(SEMA_ASW);		/* Start Compression of Buffer*/
				}break;
			}
		}
	}
#endif //0
}
