#define VIRTUOSOSIM_H "VirtuosoSim.cpp"

/***************************************************************

Virtuoso Emulator for Windows,
Copyright - Centre Spatial de Lige

***************************************************************/


/*$Header: /CVS/pacs/obsw/spu/devel/source/VirtuosoSim.h,v 1.2 2006/08/26 13:35:55 amazy Exp $*/


#ifdef CSL_SHOW_INCLUDE
#pragma message( "Including " __FILE__ ) 
#endif

#define _IFACE_H	// so "iface.h" is not included when we include "node1.h"
typedef int EVSTR;	// to fake the compiler
#define _EXTERN_DECL extern

#ifndef COMDLL_EXPORTS
#include "node1.h"
#endif


#include "windows.h"
#include "V_Macro.h"

#ifndef LINK1355_H
#include "link1355.h"
#endif

#define NB_FIFOS		10
#define NB_EVENTS		20
#define NB_SEMAPHORES	20
#define NB_RESOURCES	10
#define NB_MAILBOXES	10
#define MAX_NB_OF_RECEIVER_PER_MAILBOX	5
#define MAX_NB_OF_EMITTER_PER_MAILBOX	3
#define MAX_NB_TASKS	30
#define ANY_TASK		MAX_NB_TASKS-1

// note : max number of timer is 7, if more timers are needed, please contact alain.mazy@ulg.ac.be

#ifdef COMDLL_EXPORTS
#define COM_API __declspec(dllexport)
#else
#define COM_API __declspec(dllimport)
#endif


typedef int K_TIMER;
typedef int (*CriticalSectionFunction)(void* p_param);

typedef int K_MBOX;
typedef struct k_msg
{
	K_MBOX  mailbox;                    /* Mailbox ID */
	unsigned int	size;               /* size of message (bytes)                */
	unsigned int	info;               /* information field, free for user       */
	void   *tx_data;                    /* pointer to message data at sender side */
	void   *rx_data;                    /* pointer to message data at receiver    */
	int		tx_block;                   /* for async message posting              */
	int		tx_task;                    /* sending task                           */
	int		rx_task;                    /* receiving task                         */
	int*	transfer;                   /* internal use only                      */
} K_MSG;

COM_API int KS_MsgGet(K_MBOX mailbox, K_MSG *msg);
COM_API int KS_MsgGetW(K_MBOX mailbox, K_MSG *msg);
COM_API int KS_MsgGetWT(K_MBOX mailbox, K_MSG *msg, int timeout);
COM_API int	KS_MsgPut(K_MBOX mailbox, int prio, K_MSG * msg);
COM_API int	KS_MsgPutW(K_MBOX mailbox, int prio, K_MSG * msg);
COM_API int	KS_MsgPutWT(K_MBOX mailbox, int prio, K_MSG * msg, int timeout);
COM_API void KS_MsgDataGet(K_MSG* msg);

COM_API void InitVirtuosoSim(int firstEventID, const char* simulatorName, const char* releaseName);

COM_API void KS_CriticalSection(CriticalSectionFunction f, void *p_param);
COM_API int KS_SemaSignal(int sema);
COM_API int KS_SemaStatus(int sema);
COM_API int KS_SemaTest(int sema);
COM_API int KS_SemaTestW(int sema);
COM_API int KS_SemaTestWT(int sema, int timeout);

// This section has been added by Stefano@IFSI to (pretend to) implement missing services
// The following definition works only if the semaphore has been signaled once
#define KS_SemaReset(a)	KS_SemaTest(a);
// END of Stefano's section// END of Stefano's section


COM_API int KS_EventDisable(int event);
COM_API int KS_EventEnable(int event);
COM_API int KS_EventSignal(int event);
COM_API int KS_EventTest(int event);
COM_API int KS_EventTestW(int event);
COM_API int KS_EventTestWT(int event, int timeout);

COM_API int KS_FIFOPut(int fifo, void* data);
COM_API int KS_FIFOGet(int fifo, void* data);
COM_API int KS_FIFOGetW(int fifo, void* data);
COM_API int KS_FIFOGetWT(int fifo, void* data, int timeout);
COM_API void _KS_FIFOSetWidth(int fifo, int size); //internal function

COM_API K_TIMER* KS_LowTimerGet();
COM_API void KS_LowTimerFree(K_TIMER* p_timer);
COM_API void KS_LowTimerStart(K_TIMER* p_timer, int period, int repetitions, int sema);
COM_API void KS_LowTimerStop(K_TIMER* p_timer);

COM_API int KS_ResLock(int res);
COM_API int KS_ResLockW(int res);
COM_API	void KS_ResUnlock(int res);


#define KS_TaskSleep(a)	Sleep(a);

#define RC_OK	0
#define RC_FAIL	1
#define RC_TIME 2

// internal use only
COM_API DWORD* GetTaskIdRef(int taskID);

#define DECLARE_VIRTUOSO_TASK(taskFunctionName) \
void taskFunctionName(); \
DWORD WINAPI Thread##taskFunctionName(LPVOID lpParameter) \
{taskFunctionName(); return 1;};

#define START_VIRTUOSO_TASK(TASK_ID, taskFunctionName) \
HANDLE handleThread##taskFunctionName = CreateThread(NULL, 0, Thread##taskFunctionName, NULL, 0, GetTaskIdRef(TASK_ID));

#define SET_FIFO_WIDTH(FIFO_ID, size_in_bytes) \
_KS_FIFOSetWidth(FIFO_ID, size_in_bytes);