/***************************************************************************************
* File name: bitmodel.c
* Version.Revision: HPSPU-HLSW-OB-V12R0
*
* Purpose:  It contains the functions for building the bit model.
*
* Public Functions: None 
*
* Private Functions: None
*
* Description:	This file contains the used functions for building the bit model.
*
* Creation date & author: 28.10.2004 by  Ahmed Nabil Belbachir
* Version, Update date & author: HPSPU-HLSW-OB-V12R1 18.01.2006 by Ahmed Nabil Belbachir
****************************************************************************************/

/* The Header Files 
===================================================================================================*/

#include "bitmodel.h"
extern u4 *mem_3,*mem_4;
extern unsigned int pacs_div (unsigned int dividend,unsigned int divisor);
extern unsigned int roldiv (unsigned int dividend,unsigned int divisor);

/* constructs the bit structure */
void build_cf(bitmodel *m)
{   int i;
    u4 *cf;
    cf = m->cf;
    m->totalfreq = 0;
    for (i=1; i<=m->n; i<<=1)
    {   int j;
        for (j=i; j<=m->n; j+= i<<1)
        {   int k;
            if (m->f[j-1] & 0x8000)
                cf[j] = 0;
            else
                m->totalfreq += cf[j] = m->f[j-1];
            for (k=i>>1; k; k>>=1)
                cf[j] += cf[j-k];
        } /* end for j */
    } /* end for i */
}


/* scales the culmulative frequency tables by 0.5 and keeps nonzero values */
void scalefreq(bitmodel *m)
{   u4 *f, *endf;
    for (f=m->f, endf = f+m->n; f<endf; f++)
        *f = ((1+(*f & 0x7fff))>>1) | (*f & 0x8000);
    build_cf(m);
}


/* initialisation of bitmodel */
void initbitmodel( bitmodel *m, int n, int max_totf, int rescale)
{
	int i;

	m->n = n;
    if (max_totf < n<<1) max_totf = n<<1;
    m->max_totf = max_totf;
    m->incr = roldiv((max_totf>>1),rescale);
//    m->incr = pacs_div((max_totf>>1),rescale);
    if (m->incr < 1) m->incr = 1;
    m->f = mem_3;
    m->cf = mem_4;
    m->mask = 1;
    while ((n>>=1))
        m->mask <<=1;
	for(i=0; i<m->n; i++)
       m->f[i] = 1;
    m->totalfreq = m->n;

	while (m->totalfreq > m->max_totf)
       scalefreq(m);
    build_cf(m);
}

/* deletion of bitmodel m */
void deletebitmodel( bitmodel *m )
{  	
	u4 *pointer1, *pointer2, i;

	pointer1= m->f;
	pointer2=m->cf;
	for (i=0;i<0x8000;i++)
	{
		pointer1[i]=0;
		pointer2[i]=0;
	}

    m->n = 0;
}


/* retrieval of estimated frequencies for a symbol     */
void bitgetfreq( bitmodel *m, int sym, int *sy_f, int *lt_f)
{   int cul;
    u4 *cf;
    *sy_f = m->f[sym];
    sym++;
    cf = m->cf;
    cul = cf[sym];
    while ((sym &= (sym-1)))
        cul += cf[sym];
    *lt_f = cul - *sy_f;
}

/* update the cumulative frequency data by delta */
void bit_cfupd( bitmodel *m, int sym, int delta )
{   
	m->totalfreq += delta;
    if (m->totalfreq > m->max_totf)
        scalefreq(m);
    else
    {   
		u4 *cf;
        cf = m->cf;
        while (sym<= m->n)
        {   cf[sym] += delta;
			sym=(sym | (sym-1)) + 1;
        }
    }
}

/* update model and exclude symbol */
void bitupdate_ex( bitmodel *m, int sym )
{   int delta;
	delta = -m->f[sym];
    m->f[sym] = (m->f[sym] + m->incr) | 0x8000;
    bit_cfupd(m, sym+1, delta);
}

/* deactivate the symbol */
void bitdeactivate( bitmodel *m, int sym )
{
	bit_cfupd(m, sym+1, -m->f[sym]);
    m->f[sym] |= 0x8000;
}

/* reactivate the symbol */
void bitreactivate( bitmodel *m, int sym )
{
	m->f[sym] &= 0x7fff;
    bit_cfupd(m, sym+1, m->f[sym]);
}
