/***************************************************************************************
* File name: bitmodel.h
* Version.Revision: HPSPU-HLSW-OB-V12R0
*
* Purpose:  Header for the bit model.
*
* Public Functions: None 
*
* Private Functions: None
*
* Description:	This file contains the declared parameters for the bit model.
*
* Creation date & author: 28.10.2004 by  Ahmed Nabil Belbachir
* Version, Update date & author: HPSPU-HLSW-OB-V12R1 18.01.2006 by Ahmed Nabil Belbachir
****************************************************************************************/

/* The Header Files 
===================================================================================================*/

#ifndef BITMODEL_H
#define BITMODEL_H

/*
  bitmodel.h     headerfile for bit indexed trees probability model

*/

#include "spuAsw.h"

typedef struct {
    int n,             /* number of symbols */
        totalfreq,     /* total frequency count (without excluded symbols) */
        max_totf,      /* maximum allowed total frequency count */
        incr,          /* increment per update */
        mask;          /* initial bitmask used for search */
		u4 *f,          /* frequency for the symbol; first bit set if excluded */
        *cf;           /* array of cumulative frequencies */
} bitmodel;

/* initialisation of bitmodel                          */
/* m   bitmodel to be initialized                      */
/* n   number of symbols in that model                 */
/* max_totf  maximum allowed total frequency count     */
/* rescale  desired rescaling interval, must be <max_totf/2 */
/* init  array of int's to be used for initialisation (NULLO ok) */
void initbitmodel( bitmodel *m, int n, int max_totf, int rescale);

/* deletion of bitmodel m                              */
void deletebitmodel( bitmodel *m );


/* retrieval of estimated frequencies for a symbol     */
/* m   bitmodel to be questioned                       */
/* sym  symbol for which data is desired; must be <n   */
/* sy_f frequency of that symbol                       */
/* lt_f frequency of all smaller symbols together      */
/* the total frequency can be obtained with bit_totf   */
void bitgetfreq( bitmodel *m, int sym, int *sy_f, int *lt_f);

/* find out total frequency for a bitmodel             */
/* m   bitmodel to be questioned                       */
#define bittotf(m) ((m)->totalfreq)

/* update model and exclude symbol                     */
/* m   bitmodel to be updated                          */
/* sym  symbol that occurred (must be <n from init)    */
void bitupdate_ex( bitmodel *m, int sym );


/* deactivate symbol                                   */
/* m   bitmodel to be updated                          */
/* sym  symbol to be deactivated                       */
void bitdeactivate( bitmodel *m, int sym );

/* reactivate symbol                                   */
/* m   bitmodel to be updated                          */
/* sym  symbol to be reactivated                       */
void bitreactivate( bitmodel *m, int sym );

#endif
