#ifndef _globals_H
#define _globals_H

#include "spulib.h"
#include "spuasw.h"

#ifdef SIMULATOR
#define USE_C_IMPLEMENTATION	//on the simulator, we always use C implementation of functions and never the assembly one
#endif

//#define USE_C_IMPLEMENTATION


/****************************************************************************
Set of utility functions that exists either in assembly or in C code
****************************************************************************/

#ifdef USE_C_IMPLEMENTATION

extern void _Delta (int *buf, int words);
#define Delta(buf, size) _Delta(buf, size)

extern void _ExpandSign(int *buf, int sizeInWords);
#define ExpandSign(buf, sizeInWords) _ExpandSign(buf, sizeInWords)

extern void _MakeDataSigned(int *buf, int sizeInWords);
#define MakeDataSigned(buf, sizeInWords) _MakeDataSigned(buf, sizeInWords)

extern void _Put2ShortsIn1Word(u4* src, u4 *dest, u4 destSizeInWords);
#define Put2ShortsIn1Word(src, dest, destSizeInWords) _Put2ShortsIn1Word(src, dest, destSizeInWords)

extern void _Put2ShortsIn2Words(u4* src, u4 *dest, u4 srcSizeInWords);
#define Put2ShortsIn2Words(src, dest, srcSizeInWords) _Put2ShortsIn2Words(src, dest, srcSizeInWords)

extern void _Put2ShortsIn2WordsSigned(u4* src, u4 *dest, u4 srcSizeInWords);
#define Put2ShortsIn2WordsSigned(src, dest, srcSizeInWords) _Put2ShortsIn2WordsSigned(src, dest, srcSizeInWords)

extern void _MemCopyDmToDm(u4 *src, u4 *dest, u4 sizeInWords);
#define MemCopyDmToDm(src, dest, srcSizeInWords) _MemCopyDmToDm(src, dest, srcSizeInWords)

extern void _SelectDetectors(u4* src, u4* dest, u4* table, u4 nbPixelsPerFrame, u4 nbFrames);
#define SelectDetectors(src, dest, table, nbPixelsPerFrame, nbFrames) _SelectDetectors(src, dest, table, nbPixelsPerFrame, nbFrames)

#else	//USE_C_IMPLEMENTATION

extern void DeltaAsm(int *buf, int words);
#define Delta(buf, size) {if (size > 1) {DeltaAsm(buf, size);}}

extern void ExpandSignAsm(int *buf, int sizeInWords);
#define ExpandSign(buf, sizeInWords) {if (sizeInWords > 0) {ExpandSignAsm(buf, sizeInWords);}}

extern void MakeDataSignedAsm(int *buf, int sizeInWords);
#define MakeDataSigned(buf, sizeInWords) {if (sizeInWords > 0) {MakeDataSignedAsm(buf, sizeInWords);}}

extern void Put2ShortsIn1WordAsm(u4* src, u4 *dest, u4 destSizeInWords);
#define Put2ShortsIn1Word(src, dest, destSizeInWords) {if (destSizeInWords > 0) {Put2ShortsIn1WordAsm(src, dest, destSizeInWords);}}

extern void Put2SIn2WAsm(u4* src, u4 *dest, u4 srcSizeInWords);
#define Put2ShortsIn2Words(src, dest, srcSizeInWords) {if (srcSizeInWords > 0) {Put2SIn2WAsm(src, dest, srcSizeInWords);}}

extern void Put2SIn2WSignedAsm(u4* src, u4 *dest, u4 srcSizeInWords);
#define Put2ShortsIn2WordsSigned(src, dest, srcSizeInWords) {if (srcSizeInWords > 0) {Put2SIn2WSignedAsm(src, dest, srcSizeInWords);}}

extern void MemCopyDmToDmAsm(u4 *src, u4 *dest, u4 sizeInWords);
#define MemCopyDmToDm(src, dest, srcSizeInWords) {if (srcSizeInWords > 0) {MemCopyDmToDmAsm(src, dest, srcSizeInWords);}}

extern void SelectDetectorsAsm(u4* src, u4* dest, u4* table, u4 nbPixelsPerFrame, u4 nbFrames);
#define SelectDetectors(src, dest, table, nbPixelsPerFrame, nbFrames) SelectDetectorsAsm(src, dest, table, nbPixelsPerFrame, nbFrames)

#endif	//USE_C_IMPLEMENTATION



/****************************************************************************
Other functions
****************************************************************************/


//declared in Buf_spec.c
extern u4 BufferTransmissionModeBufferId;

//declared in Check.c
extern u4 ProcBuff[0x42000];	//a general purpose processing buffer
extern u4 Crc(u4 *start, u4 length, u4 alignment);

//declared in csw.c
extern u4 TransparentMode;

//declared in datatx.c
extern u4 DumpBuffer[256];

//declared in Dmch_cp.c
extern void DmcHeaderCompression(u4 nbFrames, u4 headerSize, COMPRESSION_MODE mode);

//declared in Dxs.c
extern void Dxs(u4 srcSizeInWords, DXS_TABLE* dxsTable, u4 nbFrames, u4 index, u4 sample, COMPRESSION_MODE mode);
extern RAW_CHANNEL_DATA RawChannels;
#ifdef SIMULATOR
extern u4	DramBuffer[0x40000];
#else
extern u4*	DramBuffer;
#endif

//declared in Fill_in.c
extern void ResetCircularBuffers();
extern void DeclareCircularBuffer(u4 bufferId, u4* start, u4* end);
extern u4 WriteInCircularBuffer(u4 bufferId, u4* src, u4 size);
extern u4 CircularBufferQuerySpace(u4 bufferId);
extern u4 CircularBufferQueryUsed(u4 bufferId);
extern u4 ReadInCircularBuffer(u4 bufferId, u4* dest, u4 size);
extern u4 ReadInCircularBufferAndPut2ShortsIn2Words(u4 bufferId, u4* dest, u4 size, u4 shortsAreSigned);
extern void CircularBufferRemoveLastWrittenValues(u4 bufferId, u4 size);
extern void CircularBufferRemove(u4 bufferId, u4 size);

//declared in Fill_out.c
extern void BuildCompressedEntity(COMPRESSION_MODE mode);
extern struct PROCESSING_RESULT Result;

//declared in Hk.c
extern struct SPU_HK_PACKET HkPacket;	//the HK Packet buffer that is used by Hk and DataTx
extern void PushDmcHeaderError(u4 error);


//declared in Llc.c
extern void CompressRawChannels(RAW_CHANNEL_DATA* rawChannels, u4 detectorType); 
extern unsigned int roldiv (unsigned int dividend,unsigned int divisor);
extern int roldivs (int dividend, int divisor);


//declared in p_proc.c
extern void PreProcessing(u4 detectorType);

//declared in Rc_Sel.c
extern RAW_CHANNEL_SELECTION RawChannelSelectionSpectro;
extern RAW_CHANNEL_SELECTION RawChannelSelectionPhoto;

//declared in SpuIo.c
extern u4 DmcLinkConnectionStatus;
#ifndef SIMULATOR
extern void Link1355Write(int channel, u4 size, u4 *data);
#endif

//declared in Supervs.c
extern u4 StoreNow;

//declared in T_S_Red.c
extern struct BUFFER_DESCRIPTION SortedDataBuffer;
extern struct BUFFER_DESCRIPTION CompressedDataBuffer;

extern void TSReductionForSpectro(COMPRESSION_MODE mode);


//declared in WatchPc.c
extern u4 CompressionSoftwareRunning;
extern u4 DataGeneratorRunning;
extern u4 BufferTransmissionModeEnabled;


//declared in Write.c
extern void RebuildDetectorSelectionTable(u4 nbPixels, u4 *src, u4 *dest);
extern void InitDetectorSelectionTables();
extern void InitTables(void);
extern int RoundingHelper[16];


extern WRT_SIM_DATA			SimulationDataParameters;
extern WRT_DET_CST_SPECTRO	DetectorConstantsSpectro;
extern WRT_DET_CST_PHOTO	DetectorConstantsPhoto;
extern u4					PreProcessingParametersSummary;
extern u4					BitMask1[32];		//array to ease some bit operations BitMask1[0] = 0x80000000, BitMask1[31] = 0x00000001
extern u4					BitMask2[32];		//array to ease some bit operations BitMask2[0] = 0x7FFFFFFF, BitMask2[31] = 0xFFFFFFFE
extern u4					WordMaskRight[32];	//array to ease some bit operations WordMaskRight[0] = 0x00000000, WordMaskRight[1] = 0x80000000, WordMaskRight[31] = 0xFFFFFFFE
extern DXS_TABLE			DxsSpectro;
extern DXS_TABLE			DxsPhotoBlue;
extern DXS_TABLE			DxsPhotoRed;
extern u4					DetectorSelectionId;

















#endif _globals_H