/*****************************************************************************
 *
 * Project name: 			Plank LFI REBA / Herschel PACS SPU
 * Product name: 			LLSW_DRV
 * Object name: 			l_edac
 * Filename: 				l_edac.h
 * Language: 				C (ADSP-21020)
 * Compiler: 				G21K - r3.3
 * Company: 				CRISA
 * Author: 					F. Torrero
 * Version: 				1
 * Creation date: 			11/May/2001
 * Last Modification date: 	04/06/2003 16:05:44
 *
 * Description ===============================================================
 *
 * This module implements functions related to EDAC management
 *
 * Change log ===============================================================
 *
 * | DATE 	  | NEW VERSION 	| AUTHOR 	  | 	REASON FOR CHANGE 	|
 * ===========================================================================
 *
 * 11/May/01 	1			 	 F.Torrero		Creation
 *
 * ****************************************************************************/ 


#ifndef L_EDAC_H
#define L_EDAC_H


/*****************************************************************************
   INCLUDES
*****************************************************************************/

#include "l_gendef.h"

/*****************************************************************************
   PROVIDED CONSTANTS
*****************************************************************************/



/*****************************************************************************
   PROVIDED TYPES
*****************************************************************************/

typedef struct {
	T_UNSIGNED_32	EDACEnaDisInPRAMStatus; /* Range: K_ENABLE, K_DISABLE */
	T_UNSIGNED_32	EDACEnaDisInEEPROMStatus; /* Range: K_ENABLE, K_DISABLE */
	T_UNSIGNED_32	EDACEnaDisInDRAMStatus; /* Range: K_ENABLE, K_DISABLE */
	T_UNSIGNED_32	DEFResetEnaDisInPMStatus; /* Range: K_ENABLE, K_DISABLE */
	T_UNSIGNED_32	DEFResetEnaDisInDMStatus; /* Range: K_ENABLE, K_DISABLE */
} T_EDAC_CONFIG_STATUS;



/*****************************************************************************
   PROVIDED VARIABLES
*****************************************************************************/


/*****************************************************************************
   DECLARATION OF PROVIDED FUNCTIONS
*****************************************************************************/


T_SR EDAC_EnaDisEDAC(T_UNSIGNED_32 HardwareUnitID,
					 T_UNSIGNED_32 MemoryAreaID,
					 T_UNSIGNED_32 EnableDisableFlag);

T_SR EDAC_EnaDisDEFReset(T_UNSIGNED_32 ProgramDataMemoryID,
						 T_UNSIGNED_32 EnableDisableFlag);

T_SR EDAC_GetEDACConfiguration(T_UNSIGNED_32 HardwareUnitID,
							   T_EDAC_CONFIG_STATUS *EDACStatus);

T_SR EDAC_GetSEFAddressInDM(T_UNSIGNED_32 *SEFAddressInDM);

T_SR EDAC_GetSEFAddressInPM(T_UNSIGNED_32 HardwareUnitID,
							T_UNSIGNED_32 *MemoryAreaIDPRAMOrEEPROM,
							T_UNSIGNED_32 *SEFAddressInPM);

T_SR EDAC_GetDEFAddressInDM(T_UNSIGNED_32 *DEFAddressInDM);

T_SR EDAC_GetDEFAddressInPM(T_UNSIGNED_32 HardwareUnitID,
							T_UNSIGNED_32 *MemoryAreaIDPRAMOrEEPROM,
							T_UNSIGNED_32 *DEFAddressInPM);

T_SR EDAC_CorrectSEFInDRAM(T_UNSIGNED_32 FailAddress);

T_SR EDAC_CorrectSEFInPRAM(T_UNSIGNED_32 FailAddress);

T_SR EDAC_CorrectSEFInEEPROM(T_UNSIGNED_32 FailAddress);

T_SR EDAC_MemoryScrubbing(T_UNSIGNED_32 HardwareUnitID);




#endif  /* L_EDAC_H */

