/*****************************************************************************
 *
 * Project name: 			Plank LFI REBA / Herschel PACS SPU
 * Product name: 			LLSW_DRV
 * Object name: 			l_smcsco
 * Filename: 				l_smcsco.h
 * Language: 				C (ADSP-21020)
 * Compiler: 				G21K - r3.3
 * Company: 				CRISA
 * Author: 					F. Torrero
 * Version: 				1
 * Creation date: 			06/Jun/2001
 * Last Modification date: 	04/06/2003 16:05:46
 *
 * Description ===============================================================
 *
 * This module implements the configuration functions for SMCS chip
 *
 * Change log ===============================================================
 *
 * | DATE 	  | NEW VERSION 	| AUTHOR 	  | 	REASON FOR CHANGE 	|
 * ===========================================================================
 *
 * 06/Jun/01 	1			 	 F.Torrero		Creation
 *
 * ****************************************************************************/ 


#ifndef L_SMCSCO_H
#define L_SMCSCO_H


/*****************************************************************************
   INCLUDES
*****************************************************************************/

#include "l_gendef.h"

/*****************************************************************************
   PROVIDED CONSTANTS
*****************************************************************************/

/* Valid transmission speed values for SMCS channel (in Mbps) */
#define K_SMCSTXSPEED_2_5		0x6
#define K_SMCSTXSPEED_5			0x5
#define K_SMCSTXSPEED_10		0x4
#define K_SMCSTXSPEED_20		0x3
#define K_SMCSTXSPEED_40		0x2
#define K_SMCSTXSPEED_80		0x1
#define K_SMCSTXSPEED_160		0x0

/* Valid transmission/reception port widths for SMCS channel (in Mbps) */
#define K_SMCSPORTWIDTH_8		0x0 
#define K_SMCSPORTWIDTH_16		0x1 
#define K_SMCSPORTWIDTH_32		0x3




/*****************************************************************************
   PROVIDED TYPES
*****************************************************************************/


/*****************************************************************************
   PROVIDED VARIABLES
*****************************************************************************/


/*****************************************************************************
   DECLARATION OF PROVIDED FUNCTIONS
*****************************************************************************/

T_SR SMCS_ConfigureSMCSNominal(T_VOID);

T_SR SMCS_SetChannelTxSpeed(T_UNSIGNED_32 Channel,
							T_UNSIGNED_32 TxSpeed);

T_SR SMCS_GetChannelTxSpeed(T_UNSIGNED_32 Channel,
							T_UNSIGNED_32 *TxSpeed);

T_SR SMCS_SetChannelTxPortWidth(T_UNSIGNED_32 Channel,
								T_UNSIGNED_32 Width);

T_SR SMCS_GetChannelTxPortWidth(T_UNSIGNED_32 Channel,
								T_UNSIGNED_32 *Width);

T_SR SMCS_SetChannelRxPortWidth(T_UNSIGNED_32 Channel,
								T_UNSIGNED_32 Width);

T_SR SMCS_GetChannelRxPortWidth(T_UNSIGNED_32 Channel,
								T_UNSIGNED_32 *Width);

T_SR SMCS_EnaDisSMCSChecksum(T_UNSIGNED_32 Channel,
							 T_UNSIGNED_32 EnableDisableFlag);

T_SR SMCS_GetSMCSChecksumStatus(T_UNSIGNED_32 Channel,
								T_UNSIGNED_32 *EnableDisableStatus);



#endif  /* L_SMCSCO_H */

