/*****************************************************************************
 *
 * Project name: 			Plank LFI REBA / Herschel PACS SPU
 * Product name: 			LLSW_DRV
 * Object name: 			l_smcsge
 * Filename: 				l_smcsge.h
 * Language: 				C (ADSP-21020)
 * Compiler: 				G21K - r3.3
 * Company: 				CRISA
 * Author: 					F. Torrero
 * Version: 				1
 * Creation date: 			20/Jul/2001
 * Last Modification date: 	04/06/2003 16:05:46
 *
 * Description ===============================================================
 *
 * This module implements the lowest level functions and general definitions
 * related to management of SMCS chip
 *
 * Change log ===============================================================
 *
 * | DATE 	  | NEW VERSION 	| AUTHOR 	  | 	REASON FOR CHANGE 	|
 * ===========================================================================
 *
 * 20/Jul/01 	1			 	 F.Torrero		Creation
 *
 * ****************************************************************************/ 


#ifndef L_SMCSGE_H
#define L_SMCSGE_H


/*****************************************************************************
   INCLUDES
*****************************************************************************/

#include "l_gendef.h"
#include "l_hwmap.h"


/*****************************************************************************
   PROVIDED CONSTANTS
*****************************************************************************/

/* Data Memory SMCS chip register addresses */
#define K_DMADD_SMCS_SICR			K_DMADDR_BASE_SMCS + 0x00
#define K_DMADD_SMCS_TRS_CTRL		K_DMADDR_BASE_SMCS + 0x01
#define K_DMADD_SMCS_RT_CTRL		K_DMADDR_BASE_SMCS + 0x02
#define K_DMADD_SMCS_ISR			K_DMADDR_BASE_SMCS + 0x04
#define K_DMADD_SMCS_IMR			K_DMADDR_BASE_SMCS + 0x08
#define K_DMADD_SMCS_COMI_CS0R		K_DMADDR_BASE_SMCS + 0x0C
#define K_DMADD_SMCS_COMI_ACR		K_DMADDR_BASE_SMCS + 0x0E
#define K_DMADD_SMCS_PRCIR			K_DMADDR_BASE_SMCS + 0x0F

#define K_DMADD_SMCS_CH1_DSM_MODR	K_DMADDR_BASE_SMCS + 0x10
#define K_DMADD_SMCS_CH1_DSM_CMDR	K_DMADDR_BASE_SMCS + 0x11
#define K_DMADD_SMCS_CH1_DSM_STAR	K_DMADDR_BASE_SMCS + 0x12
#define K_DMADD_SMCS_CH1_DSM_TSTR	K_DMADDR_BASE_SMCS + 0x13
#define K_DMADD_SMCS_CH1_ADDR		K_DMADDR_BASE_SMCS + 0x14
#define K_DMADD_SMCS_CH1_RT_ADDR	K_DMADDR_BASE_SMCS + 0x15
#define K_DMADD_SMCS_CH1_PR_STAR	K_DMADDR_BASE_SMCS + 0x16
#define K_DMADD_SMCS_CH1_CNTRL1		K_DMADDR_BASE_SMCS + 0x18
#define K_DMADD_SMCS_CH1_CNTRL2		K_DMADDR_BASE_SMCS + 0x19
#define K_DMADD_SMCS_CH1_HTID		K_DMADDR_BASE_SMCS + 0x1A
#define K_DMADD_SMCS_CH1_HCNTRL		K_DMADDR_BASE_SMCS + 0x1B
#define K_DMADD_SMCS_CH1_ESR1		K_DMADDR_BASE_SMCS + 0x1C
#define K_DMADD_SMCS_CH1_ESR2		K_DMADDR_BASE_SMCS + 0x1D
#define K_DMADD_SMCS_CH1_COMICFG	K_DMADDR_BASE_SMCS + 0x1F
#define K_DMADD_SMCS_CH1_TX_SAR		K_DMADDR_BASE_SMCS + 0x20
#define K_DMADD_SMCS_CH1_TX_EAR		K_DMADDR_BASE_SMCS + 0x22
#define K_DMADD_SMCS_CH1_TX_CAR		K_DMADDR_BASE_SMCS + 0x24
#define K_DMADD_SMCS_CH1_TX_FIFO	K_DMADDR_BASE_SMCS + 0x26
#define K_DMADD_SMCS_CH1_TX_EOPB	K_DMADDR_BASE_SMCS + 0x27
#define K_DMADD_SMCS_CH1_RX_SAR		K_DMADDR_BASE_SMCS + 0x28
#define K_DMADD_SMCS_CH1_RX_EAR		K_DMADDR_BASE_SMCS + 0x2A
#define K_DMADD_SMCS_CH1_RX_CAR		K_DMADDR_BASE_SMCS + 0x2C
#define K_DMADD_SMCS_CH1_RX_FIFO	K_DMADDR_BASE_SMCS + 0x2E
#define K_DMADD_SMCS_CH1_STAR		K_DMADDR_BASE_SMCS + 0x2F

#define K_DMADD_SMCS_CH2_DSM_MODR	K_DMADDR_BASE_SMCS + 0x30
#define K_DMADD_SMCS_CH2_DSM_CMDR	K_DMADDR_BASE_SMCS + 0x31
#define K_DMADD_SMCS_CH2_DSM_STAR	K_DMADDR_BASE_SMCS + 0x32
#define K_DMADD_SMCS_CH2_DSM_TSTR	K_DMADDR_BASE_SMCS + 0x33
#define K_DMADD_SMCS_CH2_ADDR		K_DMADDR_BASE_SMCS + 0x34
#define K_DMADD_SMCS_CH2_RT_ADDR	K_DMADDR_BASE_SMCS + 0x35
#define K_DMADD_SMCS_CH2_PR_STAR	K_DMADDR_BASE_SMCS + 0x36
#define K_DMADD_SMCS_CH2_CNTRL1		K_DMADDR_BASE_SMCS + 0x38
#define K_DMADD_SMCS_CH2_CNTRL2		K_DMADDR_BASE_SMCS + 0x39
#define K_DMADD_SMCS_CH2_HTID		K_DMADDR_BASE_SMCS + 0x3A
#define K_DMADD_SMCS_CH2_HCNTRL		K_DMADDR_BASE_SMCS + 0x3B
#define K_DMADD_SMCS_CH2_ESR1		K_DMADDR_BASE_SMCS + 0x3C
#define K_DMADD_SMCS_CH2_ESR2		K_DMADDR_BASE_SMCS + 0x3D
#define K_DMADD_SMCS_CH2_COMICFG	K_DMADDR_BASE_SMCS + 0x3F
#define K_DMADD_SMCS_CH2_TX_SAR		K_DMADDR_BASE_SMCS + 0x40
#define K_DMADD_SMCS_CH2_TX_EAR		K_DMADDR_BASE_SMCS + 0x42
#define K_DMADD_SMCS_CH2_TX_CAR		K_DMADDR_BASE_SMCS + 0x44
#define K_DMADD_SMCS_CH2_TX_FIFO	K_DMADDR_BASE_SMCS + 0x46
#define K_DMADD_SMCS_CH2_TX_EOPB	K_DMADDR_BASE_SMCS + 0x47
#define K_DMADD_SMCS_CH2_RX_SAR		K_DMADDR_BASE_SMCS + 0x48
#define K_DMADD_SMCS_CH2_RX_EAR		K_DMADDR_BASE_SMCS + 0x4A
#define K_DMADD_SMCS_CH2_RX_CAR		K_DMADDR_BASE_SMCS + 0x4C
#define K_DMADD_SMCS_CH2_RX_FIFO	K_DMADDR_BASE_SMCS + 0x4E
#define K_DMADD_SMCS_CH2_STAR		K_DMADDR_BASE_SMCS + 0x4F

#define K_DMADD_SMCS_CH3_DSM_MODR	K_DMADDR_BASE_SMCS + 0x50
#define K_DMADD_SMCS_CH3_DSM_CMDR	K_DMADDR_BASE_SMCS + 0x51
#define K_DMADD_SMCS_CH3_DSM_STAR	K_DMADDR_BASE_SMCS + 0x52
#define K_DMADD_SMCS_CH3_DSM_TSTR	K_DMADDR_BASE_SMCS + 0x53
#define K_DMADD_SMCS_CH3_ADDR		K_DMADDR_BASE_SMCS + 0x54
#define K_DMADD_SMCS_CH3_RT_ADDR	K_DMADDR_BASE_SMCS + 0x55
#define K_DMADD_SMCS_CH3_PR_STAR	K_DMADDR_BASE_SMCS + 0x56
#define K_DMADD_SMCS_CH3_CNTRL1		K_DMADDR_BASE_SMCS + 0x58
#define K_DMADD_SMCS_CH3_CNTRL2		K_DMADDR_BASE_SMCS + 0x59
#define K_DMADD_SMCS_CH3_HTID		K_DMADDR_BASE_SMCS + 0x5A
#define K_DMADD_SMCS_CH3_HCNTRL		K_DMADDR_BASE_SMCS + 0x5B
#define K_DMADD_SMCS_CH3_ESR1		K_DMADDR_BASE_SMCS + 0x5C
#define K_DMADD_SMCS_CH3_ESR2		K_DMADDR_BASE_SMCS + 0x5D
#define K_DMADD_SMCS_CH3_COMICFG	K_DMADDR_BASE_SMCS + 0x5F
#define K_DMADD_SMCS_CH3_TX_SAR		K_DMADDR_BASE_SMCS + 0x60
#define K_DMADD_SMCS_CH3_TX_EAR		K_DMADDR_BASE_SMCS + 0x62
#define K_DMADD_SMCS_CH3_TX_CAR		K_DMADDR_BASE_SMCS + 0x64
#define K_DMADD_SMCS_CH3_TX_FIFO	K_DMADDR_BASE_SMCS + 0x66
#define K_DMADD_SMCS_CH3_TX_EOPB	K_DMADDR_BASE_SMCS + 0x67
#define K_DMADD_SMCS_CH3_RX_SAR		K_DMADDR_BASE_SMCS + 0x68
#define K_DMADD_SMCS_CH3_RX_EAR		K_DMADDR_BASE_SMCS + 0x6A
#define K_DMADD_SMCS_CH3_RX_CAR		K_DMADDR_BASE_SMCS + 0x6C
#define K_DMADD_SMCS_CH3_RX_FIFO	K_DMADDR_BASE_SMCS + 0x6E
#define K_DMADD_SMCS_CH3_STAR		K_DMADDR_BASE_SMCS + 0x6F

/* Used to perform ISR workaround (dummy writing after reading ISR) */
#define K_DMADD_SMCS_DUMMY			K_DMADDR_BASE_SMCS + 0x7F



/* Number of memory position between two similar registers of consecutive
SMCS channels */
#define K_OFFSET_BETWEEN_SMCSCHANNELS	0x20


/* Selected bit groups, used to mask and preserve these bits */
/* CHx_DSM_MODR register */
#define K_BMASK_CHx_DSM_MODR_TXSPEED		0x7 /* bits 2:0 */
/* CHx_COMICFG register */
#define K_BMASK_CHx_COMICFG_TXPORTWIDTH		0x3 /* bits 1:0 */
#define K_BMASK_CHx_COMICFG_RXPORTWIDTH		0x30 /* bits 5:4 */


/* Positions of selected group of bits (= number of bytes to right-shift
	in order to have the group starting at position 0). Used  in shifting 
	operations and in set/clear bit operations. */
/* CHx_COMICFG register */
#define K_BPOS_CHx_COMICFG_EOPID			2
#define K_BPOS_CHx_COMICFG_RXPORTWIDTH		4
/* CHx_CNTRL1 register */
#define K_BPOS_CHx_CNTRL1_CHECKSUMENA		4
/* CHx_DSM_CMDR register */
#define K_BPOS_CHx_DSM_CMDR_STOPLINK		0
#define K_BPOS_CHx_DSM_CMDR_STARTTXNODE		1
/* CHx_DSM_STAR register */
#define K_BPOS_CHx_DSM_STAR_TXLINKRUNNIN	0
#define K_BPOS_CHx_DSM_STAR_DISCONERR		1
#define K_BPOS_CHx_DSM_STAR_PARITYERR		2
#define K_BPOS_CHx_DSM_STAR_NULLRECEIVED	3
#define K_BPOS_CHx_DSM_STAR_FCTRECEIVED		4
/* CHx_ESR1 register */
#define K_BPOS_CHx_ESR1_CHECKSUMERR			4


/* SMCS transmission/reception channels */
#define K_SMCSCHANNEL_1		1
#define K_SMCSCHANNEL_2		2
#define K_SMCSCHANNEL_3		3



/*****************************************************************************
   PROVIDED TYPES
*****************************************************************************/


/*****************************************************************************
   PROVIDED VARIABLES
*****************************************************************************/


/*****************************************************************************
   DECLARATION OF PROVIDED FUNCTIONS
*****************************************************************************/

T_SR SMCS_ReadSMCSReg(T_UNSIGNED_32 Register,
					  T_UNSIGNED_32 *RegisterValue);

T_SR SMCS_WriteSMCSReg(T_UNSIGNED_32 Register,
					   T_UNSIGNED_32 RegisterValue);

T_UNSIGNED_32 SMCS_GetRegAddressForChannel(T_UNSIGNED_32 Channel,
										   T_UNSIGNED_32 AddressChannel1);

T_VOID SMCS_WriteBitInSMCSReg(T_UNSIGNED_32 Register,
							  T_UNSIGNED_32 BitPosition,
							  T_UNSIGNED_32 BitValue);

T_BOOLEAN SMCS_CheckBitInSMCSReg(T_UNSIGNED_32 Register,
								 T_UNSIGNED_32 BitPosition);


#endif  /* L_SMCSGE_H */

