/*****************************************************************************
 *
 * Project name: 			Plank LFI REBA / Herschel PACS SPU
 * Product name: 			LLSW_DRV
 * Object name: 			l_smcsin
 * Filename: 				l_smcsin.h
 * Language: 				C (ADSP-21020)
 * Compiler: 				G21K - r3.3
 * Company: 				CRISA
 * Author: 					F. Torrero
 * Version: 				1
 * Creation date: 			06/Aug/2001
 * Last Modification date: 	04/06/2003 16:05:46
 *
 * Description ===============================================================
 *
 * This module implements SMCS sub-interrupts management functions
 *
 * Change log ===============================================================
 *
 * | DATE 	  | NEW VERSION 	| AUTHOR 	  | 	REASON FOR CHANGE 	|
 * ===========================================================================
 *
 * 06/Aug/01 	1			 	 F.Torrero		Creation
 *
 * ****************************************************************************/ 


#ifndef L_SMCSIN_H
#define L_SMCSIN_H


/*****************************************************************************
   INCLUDES
*****************************************************************************/

#include "l_gendef.h"

/*****************************************************************************
   PROVIDED CONSTANTS
*****************************************************************************/

/* Bit positions within ISR register */
#define K_BPOS_SMCSINT_CH1_PAR_DIS_ERR		0
#define K_BPOS_SMCSINT_CH1_ERROR			1
#define K_BPOS_SMCSINT_CH1_DATA_TXED		2
#define K_BPOS_SMCSINT_CH1_DATA_RXED		3
#define K_BPOS_SMCSINT_CH1_EOP_RXED			4
#define K_BPOS_SMCSINT_CH1_RSTCOM_RXED		5
#define K_BPOS_SMCSINT_CH1_TXFIFOEMPTY		6
#define K_BPOS_SMCSINT_CH1_TXFIFOFULL		7
#define K_BPOS_SMCSINT_CH1_RXFIFONOTEM		8
#define K_BPOS_SMCSINT_CH1_RXFIFOFULL		9

#define K_BPOS_SMCSINT_CH2_PAR_DIS_ERR		10
#define K_BPOS_SMCSINT_CH2_ERROR			11
#define K_BPOS_SMCSINT_CH2_DATA_TXED		12
#define K_BPOS_SMCSINT_CH2_DATA_RXED		13
#define K_BPOS_SMCSINT_CH2_EOP_RXED			14
#define K_BPOS_SMCSINT_CH2_RSTCOM_RXED		15
#define K_BPOS_SMCSINT_CH2_TXFIFOEMPTY		16
#define K_BPOS_SMCSINT_CH2_TXFIFOFULL		17
#define K_BPOS_SMCSINT_CH2_RXFIFONOTEM		18
#define K_BPOS_SMCSINT_CH2_RXFIFOFULL		19

#define K_BPOS_SMCSINT_CH3_PAR_DIS_ERR		20
#define K_BPOS_SMCSINT_CH3_ERROR			21
#define K_BPOS_SMCSINT_CH3_DATA_TXED		22
#define K_BPOS_SMCSINT_CH3_DATA_RXED		23
#define K_BPOS_SMCSINT_CH3_EOP_RXED			24
#define K_BPOS_SMCSINT_CH3_RSTCOM_RXED		25
#define K_BPOS_SMCSINT_CH3_TXFIFOEMPTY		26
#define K_BPOS_SMCSINT_CH3_TXFIFOFULL		27
#define K_BPOS_SMCSINT_CH3_RXFIFONOTEM		28
#define K_BPOS_SMCSINT_CH3_RXFIFOFULL		29

/* Bit masks for ISR register */
#define K_BMASK_SMCSINT_CH1_PAR_DIS_ERR		0x1
#define K_BMASK_SMCSINT_CH1_ERROR			0x2
#define K_BMASK_SMCSINT_CH1_DATA_TXED		0x4
#define K_BMASK_SMCSINT_CH1_DATA_RXED		0x8
#define K_BMASK_SMCSINT_CH1_EOP_RXED		0x10
#define K_BMASK_SMCSINT_CH1_RSTCOM_RXED		0x20
#define K_BMASK_SMCSINT_CH1_TXFIFOEMPTY		0x40
#define K_BMASK_SMCSINT_CH1_TXFIFOFULL		0x80
#define K_BMASK_SMCSINT_CH1_RXFIFONOTEM		0x100
#define K_BMASK_SMCSINT_CH1_RXFIFOFULL		0x200

#define K_BMASK_SMCSINT_CH2_PAR_DIS_ERR		0x400
#define K_BMASK_SMCSINT_CH2_ERROR			0x800
#define K_BMASK_SMCSINT_CH2_DATA_TXED		0x1000
#define K_BMASK_SMCSINT_CH2_DATA_RXED		0x2000
#define K_BMASK_SMCSINT_CH2_EOP_RXED		0x4000
#define K_BMASK_SMCSINT_CH2_RSTCOM_RXED		0x8000
#define K_BMASK_SMCSINT_CH2_TXFIFOEMPTY		0x10000
#define K_BMASK_SMCSINT_CH2_TXFIFOFULL		0x20000
#define K_BMASK_SMCSINT_CH2_RXFIFONOTEM		0x40000
#define K_BMASK_SMCSINT_CH2_RXFIFOFULL		0x80000

#define K_BMASK_SMCSINT_CH3_PAR_DIS_ERR		0x100000
#define K_BMASK_SMCSINT_CH3_ERROR			0x200000
#define K_BMASK_SMCSINT_CH3_DATA_TXED		0x400000
#define K_BMASK_SMCSINT_CH3_DATA_RXED		0x800000
#define K_BMASK_SMCSINT_CH3_EOP_RXED		0x1000000
#define K_BMASK_SMCSINT_CH3_RSTCOM_RXED		0x2000000
#define K_BMASK_SMCSINT_CH3_TXFIFOEMPTY		0x4000000
#define K_BMASK_SMCSINT_CH3_TXFIFOFULL		0x8000000
#define K_BMASK_SMCSINT_CH3_RXFIFONOTEM		0x10000000
#define K_BMASK_SMCSINT_CH3_RXFIFOFULL		0x20000000

#define K_BMASK_SMCSINT_ALL					0x3FFFFFFF



/*****************************************************************************
   PROVIDED TYPES
*****************************************************************************/


/*****************************************************************************
   PROVIDED VARIABLES
*****************************************************************************/


/*****************************************************************************
   DECLARATION OF PROVIDED FUNCTIONS
*****************************************************************************/

T_SR SMCS_MaskUnmaskSMCSSubInt(T_UNSIGNED_32 InterruptBitMask,
							   T_UNSIGNED_32 MaskUnmaskFlag);

T_SR SMCS_GetSMCSIMR(T_UNSIGNED_32 *InterruptMaskRegValue);

T_SR SMCS_GetSMCSIPR(T_UNSIGNED_32 *InterruptPendingRegValue);

T_SR SMCS_ClearSMCSIPR(T_VOID);



#endif  /* L_SMCSIN_H */

