/***************************************************************************************
* File name: Peak_Up.c
* Version.Revision: HPSPU-HLSW-OB-V3R0
*
* Purpose:  Acknowledges the Peak-up Command.			 
*
* Public Functions: None
*
* Private Functions: None
*
* Description:	This Program Acknowledges the Reception of the Peak-Up Command
*				and Starts the Appropriate Program.
*
* Creation date & author: 28.03.2001 by  Ahmed Nabil Belbachir
* Version, Update date & author: HPSPU-HLSW-OB-V3R0 07.03.2003 by Ahmed Nabil Belbachir
****************************************************************************************/

/*The Header Files 
===================================================================================================*/

#ifdef SIMULATOR
#include "virtuosoSim.h"
#else // SIMULATOR
#include "node1.h"
#include "k_struct.h"
#endif

#include "spulib.h"

/*================================================================================================*/
/*Declaration of the Used Functions and Global Variables
==================================================================================================*/


/*================================================================================================*/
/* The Main Program
==================================================================================================*/

void PerformPeakUp(u4 buf[])
{
	/* Declaration of Local Variables
	===============================================================================================*/

	struct FIFO_TRANSMIT_MSG fifoTransmitMsg;
 	u4 sid=buf[1]&0xFFFF;
	
	/*=============================================================================================*/

	if (sid!=0) /* Negative Acknowledgment:  Invalid Structure ID */
	{
		fifoTransmitMsg.Header = K_NACK_PERFORM | K_INVALID_SID ;
	  	fifoTransmitMsg.Block1.NackParameter = sid;
		
		KS_FIFOPut(FIFO_TRANSMIT, &fifoTransmitMsg); /* Send the Response Packet to the DPU */
	}
	else /* Positive Acknowledgment */		
	{
		fifoTransmitMsg.Header = K_PACK_PERFORM;
		    
		KS_FIFOPut(FIFO_TRANSMIT, &fifoTransmitMsg); /* Send the Response Packet to the DPU */

		/* Signal the Start of the Peak-up Program */

		PeakUpRunning=K_YES;
		PeakUpAndBbcRunning =K_YES;		
		KS_TaskSleep(20);
		CompressionSoftwareRunning=K_YES;
	}
}
/* End of the Main Program 
===================================================================================================*/
