/***************************************************************************************
* File name: Perform.c
* Purpose:  Handles the Perform Commands.
*
* Description:	This Program Acknowledges the Reception of the Perform Command
*				and Starts the Appropriate Acknowledgment Task According to the
*				Received Activity ID.
****************************************************************************************/

#ifdef SIMULATOR
#include "virtuosoSim.h"
#else // SIMULATOR
#include "node1.h"
#include "k_struct.h"
#endif

#include "spulib.h"

/*=================================================================================================*/
/*Functions declared in this file
===================================================================================================*/

void Perform(u4 buf[]);

/*=================================================================================================*/
/*External function and variables
===================================================================================================*/

extern void PerformCopyToEeprom(u4 buf[]);
extern void PerformWarmReset(u4 buf[]);
extern void PerformRawChannelSelection(u4 buf[]);
extern void PerformStartStop(u4 buf[]);
extern void PerformSpuTestMode(u4 buf[]);
extern void PerformConnectToDmc(u4 buf[]);

/*=================================================================================================*/

/*
This function handles the perform commands
*/
void Perform(u4 buf[])
{
	struct FIFO_TRANSMIT_MSG fifoTransmitMsg;	//msg to communicate with the data tx task
	u4 activityID;
	
	/*=============================================================================================*/

	/* Read the Activity ID */
   	activityID = buf[1]>>16;
	
	switch (activityID)/* Execute the Required Activity */	
	{
	case 0x04 :   /* Case of Copy data from RAM to EEPROM */ 
		PerformCopyToEeprom(buf);
		break;
	case 0x05 :   /* Case of Command of the Warm Reset */ 
		PerformWarmReset(buf);
		break;
	case 0x06 :   /* Case of Raw Channel Selection Command */ 
		PerformRawChannelSelection(buf);
		break;
	case 0x07 :    /* Case of Stop Command */
		PerformStartStop(buf);
		break;
	case 0x08 :   /* Case of Start Command */
		PerformStartStop(buf);
		break;
	case 0x0A :   /* Case of the SPU Test Mode for Photometry */
		PerformSpuTestMode(buf);
		break;
	case 0x0B : /* Case of the SPU Test Mode for Spectroscopy */
		PerformSpuTestMode(buf);
		break;
	case 0x10 : /* Case of Connect to DEC/MEC Command */
		PerformConnectToDmc(buf);
		break;
	default :  /* Invalid Activity ID */
		{	
			fifoTransmitMsg.Header = K_NACK_PERFORM | K_INVALID_ACTIVITY_ID; /* Invalid ActivityID */
			fifoTransmitMsg.Block1.NackParameter = activityID;
			
			KS_FIFOPut(FIFO_TRANSMIT, &fifoTransmitMsg); /* Send the Response Packet to the DPU */
		}break;
	}									
}
