/***************************************************************************************
* File name: rangecod.c
* Version.Revision: HPSPU-HLSW-OB-V12R0
*
* Purpose:  It contains the functions of the range coders.
*
* Public Functions: None 
*
* Private Functions: None
*
* Description:	This file contains the used range coders functions.
*
* Creation date & author: 28.10.2004 by  Ahmed Nabil Belbachir
* Version, Update date & author: HPSPU-HLSW-OB-V12R1 18.01.2006 by Ahmed Nabil Belbachir
****************************************************************************************/

/* The Header Files 
===================================================================================================*/


#include "rangecod.h"
extern unsigned int pacs_div (unsigned int dividend,unsigned int divisor);
extern unsigned int roldiv (unsigned int dividend,unsigned int divisor);


/* SIZE OF RANGE ENCODING CODE VALUES. */

#define CODE_BITS 32
#define Top_value ((code_value)1 << (CODE_BITS-1))
#define Top_value1 (Top_value-1)
#define EXTRA_BITS (((CODE_BITS-2)&7) + 1)
#define Bottom_value (Top_value >> 8)
#define Bottom_value1 (Bottom_value-1)
u4 SHIFT_BITS= (CODE_BITS - 9);
u4 SHIFT_BITS1= (0xFF<<(CODE_BITS - 9));

extern u4 *out_now, point;
rangecoder RNGC;
#define M_outbyte(a) (out_now[point++]=a) //outbyte(rc,a)


void start_encoding(rangecoder *rc, u4 c, int initlength)
{   RNGC.low = 0;                /* Full code range */
    RNGC.range = Top_value;
    RNGC.buffer = c;
    RNGC.help = 0;               /* No bytes to follow */
    RNGC.bytecount = initlength;
}


/* normalization.  */
void enc_normalize( rangecoder *rc )
{ 
	while(RNGC.range <= Bottom_value)     /* do we need renormalisation?  */
    {
		if (RNGC.low < SHIFT_BITS1)  /* no carry possible --> output */
        {   M_outbyte(RNGC.buffer);
            for(; RNGC.help; RNGC.help--)
                M_outbyte(0xff);
            RNGC.buffer = (RNGC.low >> SHIFT_BITS);
        } else if (RNGC.low & Top_value) /* carry now, no future carry */
        {   M_outbyte(RNGC.buffer+1);
            for(; RNGC.help; RNGC.help--)
                M_outbyte(0);
            RNGC.buffer = (RNGC.low >> SHIFT_BITS);
        } else                           /* passes on a potential carry */
            RNGC.help++;
        RNGC.range <<= 8;
        RNGC.low = (RNGC.low<<8) & (Top_value1);
        RNGC.bytecount++;
    }
}


/* Encode a symbol using frequencies                         */

void encode_freq( rangecoder *rc, u4 sy_f, u4 lt_f, u4 tot_f )
{	u4 r;
	enc_normalize( rc );
	r = roldiv (RNGC.range, tot_f);
	RNGC.low += (r * lt_f);
    RNGC.range = r * sy_f;
}

void encode_shift( rangecoder *rc, u4 sy_f, u4 lt_f, u4 shift )
{	u4 r;
	enc_normalize( rc );
	r = RNGC.range >> shift;
	RNGC.low += (r * lt_f);
	RNGC.range = r * sy_f;
}


/* Finish encoding                                           */
u4 done_encoding( rangecoder *rc )
{   u4 tmp;
    enc_normalize(rc);     /* now we have a normalized state */
    RNGC.bytecount += 5;
    if ((RNGC.low & (Bottom_value1)) < (RNGC.bytecount>>1))
       tmp = RNGC.low >> SHIFT_BITS;
    else
       tmp = (RNGC.low >> SHIFT_BITS) + 1;
    if (tmp > 0xff) /* we have a carry */
    {   M_outbyte(RNGC.buffer+1);
        for(; RNGC.help; RNGC.help--)
            M_outbyte(0);
    } else  /* no carry */
    {   M_outbyte(RNGC.buffer);
        for(; RNGC.help; RNGC.help--)
            M_outbyte(0xff);
    }
    M_outbyte(tmp & 0xff);
    M_outbyte((RNGC.bytecount>>16) & 0xff);
    M_outbyte((RNGC.bytecount>>8) & 0xff);
    M_outbyte(RNGC.bytecount & 0xff);
    return RNGC.bytecount;
}
