/*****************************************************************************************
* File name: WatchPc.c
* Purpose:  Handles the DPU Commands
*
* Description:	The Watch Process Program is the Task with Highest Priority.
*				It listens to the commands from DPU Link. 
*****************************************************************************************/

#ifdef SIMULATOR
#include "virtuosoSim.h"
#else // SIMULATOR
#include "node1.h"
#include "k_struct.h"
#endif
#include "spulib.h"


/*=================================================================================================*/
/*Functions declared in this file
===================================================================================================*/

void WatchDpu(void);

/*=================================================================================================*/
/*Global variables declared in this file
===================================================================================================*/

u4 CompressionSoftwareRunning;
u4 DataGeneratorRunning;
u4 BufferTransmissionModeEnabled;

/*=================================================================================================*/
/*External function and variables
===================================================================================================*/

extern void Load(u4 buf[]);
extern void Dump(u4 buf[]);
extern void Check(u4 buf[]);
extern void Perform(u4 buf[]);
extern void Write(u4 buf[]);

#ifndef SIMULATOR 
extern void LinkSmcsRead(int device);
extern int ChannelReceiveAddress[3];
#endif

/*================================================================================================*/

/*
This is the WATCH_DPU task entry point.
*/
void WatchDpu(void)
{	
	struct FIFO_TRANSMIT_MSG fifoTransmitMsg;	//msg to communicate with the data tx task
	u4 buf[272], commandId;
	
	/*============================================================================================*/
	/* Initialization of the Parameters for the SPU Idle Mode
	==============================================================================================*/
	
	CompressionSoftwareRunning = K_NO;		/* CSW Stopped */
	DataGeneratorRunning = K_NO;			/* Data Generators Stopped */
	BufferTransmissionModeEnabled = K_NO;	/* Buffer Transmission Mode Disabled */
	
	/*============================================================================================*/
	/* Initialization of the DXS Tables and Bit Mask tables at StartUp
	==============================================================================================*/
	
	InitDetectorSelectionTables();
	InitTables();
	
	while(1)
	{
		/* Listen to the DPU Link
		=========================================================================================*/
#ifdef SIMULATOR 
		static int packetCounter = 0;
		static int ListenToDpu = 1;
		packetCounter++;
		if (ListenToDpu)
		{
			Link1355Read(K_SMCSCHANNEL_1, 272, (void*)buf);
		}
#else
		LinkSmcsRead(K_SMCSCHANNEL_1);
		MemCopyDmToDm((u4 *)ChannelReceiveAddress[K_SMCSCHANNEL_1-1],buf,100);
#endif	

		commandId = buf[0];
		commandId= (commandId>>16)&0xFFFF;
			
		if (CompressionSoftwareRunning == K_YES)
		{
			//if the compression software is running, the only accepted command is the stop CSW
			//all other commands will be rejected
			u4 activityID = buf[1]>>16;
			if ((commandId == 4) && (activityID == 0x07))
			{
				Perform(buf);
			}
			else //for every other command, send a NACK
			{
				fifoTransmitMsg.Header = K_NACK_INVALID_CMD | K_INVALID_CMD_ID; /* Invalid Command ID */
				fifoTransmitMsg.Block1.NackParameter = commandId;
				KS_FIFOPut(FIFO_TRANSMIT, &fifoTransmitMsg); /* Send the Response Packet to the DPU */
			}
		}
		else // when compression software is not running, accept all commands
		{
			switch(commandId) 
			{
			case 1:/* Case of the Load Command */
				Load(buf);
				break;

			case 2:	/* Case of the Dump Command */
				Dump(buf);
				break;

			case 3: /* Case of the Check Command */
				Check(buf);
				break;

			case 4: /* Case of the Perform Command */
				Perform(buf);
				break;

			case 6:	/* Case of the Write Command */	
				Write(buf);
				break;

			default:/* Case of the Invalid Command ID */
				{
					struct FIFO_TRANSMIT_MSG fifoTransmitMsg;

					fifoTransmitMsg.Header = K_NACK_INVALID_CMD | K_INVALID_CMD_ID; /* Invalid Command ID */
					fifoTransmitMsg.Block1.NackParameter = commandId;
					KS_FIFOPut(FIFO_TRANSMIT, &fifoTransmitMsg); /* Send the Response Packet to the DPU */
				}break;
			}	
		}
	}
}
